@@ -78,14 +78,22 @@ pdl_error pdl__ensure_trans(pdl_trans *trans,int what,int *wd)
 }
 
 pdl *pdl_null() {
-	PDL_Indx d[1] = {0};
+	PDL_Anyval zero = { PDL_B, {0} };
 	pdl *it = pdl_pdlnew();
 	if (!it) return it;
-	PDL_Anyval zero = { PDL_B, {0} };
-	pdl_makescratchhash(it, zero);
-	pdl_error PDL_err = pdl_setdims(it,d,1);
+	pdl_error PDL_err = pdl_makescratchhash(it, zero);
 	if (PDL_err.error) { pdl_destroy(it); return NULL; }
-	it->state |= PDL_NOMYDIMS;
+	return it;
+}
+
+pdl *pdl_scalar(PDL_Anyval anyval) {
+	pdl *it = pdl_pdlnew();
+	if (!it) return it;
+	pdl_error PDL_err = pdl_makescratchhash(it, anyval);
+	if (PDL_err.error) { pdl_destroy(it); return NULL; }
+	it->threadids[0] = it->ndims = 0; /* 0 dims in a scalar */
+	it->state &= ~(PDL_ALLOCATED|PDL_NOMYDIMS); /* size changed, has dims */
+	it->nvals = 1;            /* 1 val  in a scalar */
 	return it;
 }
 
@@ -106,8 +114,10 @@ pdl_error pdl_grow(pdl* a, PDL_Indx newsize) {
    STRLEN len;
    nbytes = ((STRLEN) newsize) * pdl_howbig(a->datatype);
    ncurr  = a->datasv ? SvCUR((SV *)a->datasv) : 0;
-   if (ncurr == nbytes)
+   if (ncurr == nbytes) {
+      a->state |= PDL_ALLOCATED;
       return PDL_err;    /* Nothing to be done */
+   }
    if(a->state & PDL_DONTTOUCHDATA) {
       return pdl_make_error_simple(PDL_EUSERERROR, "Trying to touch data of an untouchable (mmapped?) pdl");
    }
@@ -144,13 +154,11 @@ pdl_error pdl_allocdata(pdl *it) {
 }
 
 pdl* pdl_pdlnew() {
-     int i;
      pdl* it;
      it = (pdl*) malloc(sizeof(pdl));
      if (!it) return it;
      memset(it, 0, sizeof(pdl));
      it->magicno = PDL_MAGICNO;
-     it->state = 0;
      it->datatype = 0;
      it->trans_parent = NULL;
      it->vafftrans = NULL;
@@ -158,12 +166,15 @@ pdl* pdl_pdlnew() {
      it->datasv = 0;
      it->data = 0;
      it->has_badvalue = 0;
+     it->state = PDL_NOMYDIMS;
      it->dims = it->def_dims;
+     it->nvals = it->dims[0] = 0;
      it->dimincs = it->def_dimincs;
-     it->ndims = 0;
+     it->dimincs[0] = 1;
      it->nthreadids = 1;
      it->threadids = it->def_threadids;
-     it->threadids[0] = 0;
+     it->threadids[0] = it->ndims = 1;
+     PDL_Indx i;
      for(i=0; i<PDL_NCHILDREN; i++) {it->child_transes.trans[i]=NULL;}
      it->child_transes.next = NULL;
      it->magic = 0;
@@ -521,33 +532,39 @@ pdl_error pdl_reallocthreadids(pdl *it, PDL_Indx n) {
 	return PDL_err;
 }
 
-/* Recalculate default increments and grow the PDL data */
-
+/* Recalculate default increments */
 void pdl_resize_defaultincs(pdl *it) {
-	PDL_Indx inc = 1;
-	unsigned int i=0;
-	for(i=0; i<it->ndims; i++) {
-		it->dimincs[i] = inc; inc *= it->dims[i];
-	}
-	it->nvals = inc;
-        it->state &= ~PDL_ALLOCATED; /* Need to realloc when phys */
+    PDL_Indx inc = 1, i = 0;
+    for(i=0; i<it->ndims; i++) {
+	it->dimincs[i] = inc; inc *= it->dims[i];
+    }
+    if (it->nvals != inc) /* Need to realloc only if nvals changed */
+	it->state &= ~PDL_ALLOCATED;
+    it->nvals = inc;
 }
 
 /* Init dims & incs - if *incs is NULL ignored (but space is always same for both)  */
-
 pdl_error pdl_setdims(pdl* it, PDL_Indx * dims, PDL_Indx ndims) {
-   pdl_error PDL_err = {0, NULL, 0};
-   PDL_Indx i;
-   PDLDEBUG_f(printf("pdl_setdims %p: ", it));PDLDEBUG_f(pdl_print_iarr(dims, ndims));PDLDEBUG_f(printf("\n"));
-   PDL_RETERROR(PDL_err, pdl_changesoon(it));
-   PDL_RETERROR(PDL_err, pdl_reallocdims(it,ndims));
-   for(i=0; i<ndims; i++) it->dims[i] = dims[i];
-   pdl_resize_defaultincs(it);
-   PDL_RETERROR(PDL_err, pdl_reallocthreadids(it,1));
-   it->threadids[0] = ndims;
-   it->state &= ~PDL_NOMYDIMS;
-   PDL_RETERROR(PDL_err, pdl_changed(it,PDL_PARENTDIMSCHANGED|PDL_PARENTDATACHANGED,0));
-   return PDL_err;
+  pdl_error PDL_err = {0, NULL, 0};
+  PDLDEBUG_f(printf("pdl_setdims %p: ", it));PDLDEBUG_f(pdl_print_iarr(dims, ndims));PDLDEBUG_f(printf("\n"));
+  PDL_Indx i, old_nvals = it->nvals, new_nvals = 1;
+  for (i=0; i<ndims; i++) new_nvals *= dims[i];
+  int what = (old_nvals == new_nvals) ? 0 : PDL_PARENTDATACHANGED;
+  if ((it->state & PDL_NOMYDIMS) || ndims != it->ndims)
+    what |= PDL_PARENTDIMSCHANGED;
+  else
+    for (i=0; i<ndims; i++)
+      if (dims[i] != it->dims[i]) { what |= PDL_PARENTDIMSCHANGED; break; }
+  if (!what) { PDLDEBUG_f(printf("pdl_setdims NOOP\n")); return PDL_err; }
+  PDL_RETERROR(PDL_err, pdl_changesoon(it));
+  PDL_RETERROR(PDL_err, pdl_reallocdims(it,ndims));
+  for(i=0; i<ndims; i++) it->dims[i] = dims[i];
+  pdl_resize_defaultincs(it);
+  PDL_RETERROR(PDL_err, pdl_reallocthreadids(it,1));
+  it->threadids[0] = ndims;
+  it->state &= ~PDL_NOMYDIMS;
+  PDL_RETERROR(PDL_err, pdl_changed(it,what,0));
+  return PDL_err;
 }
 
 /* This is *not* careful! */
