@@ -35,42 +35,28 @@ size_t pdl_howbig (int datatype) {
 }
 
 /* Make a scratch dataspace for a scalar pdl */
-
-void pdl_makescratchhash(pdl *ret, PDL_Anyval data) {
+pdl_error pdl_makescratchhash(pdl *ret, PDL_Anyval data) {
+  pdl_error PDL_err = {0, NULL, 0};
   STRLEN n_a;
-  SV *dat; PDL_Indx fake[1];
-  
-  /* Compress to smallest available type.  */
-  ret->datatype = data.type;
-
   /* Create a string SV of apropriate size.  The string is arbitrary
    * and just has to be larger than the largest datatype.   */
-  dat = newSVpvn("                                ",pdl_howbig(ret->datatype));
-  
-  ret->data = SvPV(dat,n_a);
-  ret->datasv = dat;
+  ret->datasv = newSVpvn("                                ",pdl_howbig(ret->datatype = data.type));
+  ret->data = SvPV((SV *)ret->datasv,n_a);
   /* Refcnt should be 1 already... */
-    
   /* Make the whole pdl mortal so destruction happens at the right time.
    * If there are dangling references, pdlapi.c knows not to actually
    * destroy the C struct. */
   sv_2mortal(getref_pdl(ret));
-  
-  pdl_setdims(ret, fake, 0); /* 0 dims in a scalar */
-  ret->nvals = 1;            /* 1 val  in a scalar */
-  
   /* NULLs should be ok because no dimensions. */
-  pdl_set(ret->data, ret->datatype, NULL, NULL, NULL, 0, 0, data);
-  
+  PDL_RETERROR(PDL_err, pdl_set(ret->data, ret->datatype, NULL, NULL, NULL, 0, 0, data));
+  return PDL_err;
 }
 
-
 /*
   "Convert" a perl SV into a pdl (alright more like a mapping as
    the data block is not actually copied in the most common case
    of a single scalar) scalars are automatically converted to PDLs.
 */
-
 pdl* pdl_SvPDLV ( SV* sv ) {
 
    pdl* ret;
@@ -84,12 +70,8 @@ pdl* pdl_SvPDLV ( SV* sv ) {
    if ( !SvROK(sv) ) {
       /* The scalar is not a ref, so we can use direct conversion. */
       PDL_Anyval data;
-      ret = pdl_pdlnew();  /* Scratch pdl */
-      if (!ret) return ret;
-      /* Scratch hash for the pdl :( - slow but safest. */
       ANYVAL_FROM_SV(data, sv, TRUE, -1);
-      pdl_makescratchhash(ret, data);
-      return ret;
+      return pdl_scalar(data);
    } /* End of scalar case */
 
    if(sv_derived_from(sv, "Math::Complex")) {
@@ -110,12 +92,9 @@ pdl* pdl_SvPDLV ( SV* sv ) {
         PUTBACK;
       }
       FREETMPS; LEAVE;
-      ret = pdl_pdlnew();  /* Scratch pdl */
-      if (!ret) return ret;
       data.type = PDL_CD;
       data.value.C = (PDL_CDouble)(vals[0] + I * vals[1]);
-      pdl_makescratchhash(ret, data);
-      return ret;
+      return pdl_scalar(data);
    }
 
    /* If execution reaches here, then sv is NOT a scalar
@@ -520,12 +499,13 @@ pdl* pdl_from_array(AV* av, AV* dims, int dtype, pdl* dest_pdl)
   if (dest_pdl == NULL)
      dest_pdl = pdl_pdlnew();
   if (!dest_pdl) return dest_pdl;
-  pdl_setdims (dest_pdl, dest_dims, ndims);
+  pdl_error err = pdl_setdims (dest_pdl, dest_dims, ndims);
+  if (err.error) return NULL;
   if (dtype == -1) {
     dtype = _detect_datatype(av);
   }
   dest_pdl->datatype = dtype;
-  pdl_error err = pdl_allocdata (dest_pdl);
+  err = pdl_allocdata (dest_pdl);
   if (err.error) return NULL;
   err = pdl_make_physical(dest_pdl);
   if (err.error) return NULL;
