@@ -41,7 +41,7 @@
 #undef warn
 #define warn PDL_CORE_(pdl_warn)
 
-void pdl_makescratchhash(pdl *ret, PDL_Anyval data);
+pdl_error pdl_makescratchhash(pdl *ret, PDL_Anyval data);
 PDL_Indx av_ndcheck(AV* av, AV* dims, int level, int *datalevel);
 pdl* pdl_from_array(AV* av, AV* dims, int type, pdl* p);
 PDL_Anyval pdl_at( void* x, int datatype, PDL_Indx* pos, PDL_Indx* dims, /* Value at x,y,z,... */
@@ -78,6 +78,7 @@ void pdl_dump_trans_fixspace(pdl_trans *it, int nspac);
   X(pdlnew, pdl*, ()) \
   X(destroy, pdl_error, (pdl *it)) \
   X(null, pdl*, ()) \
+  X(scalar, pdl*, (PDL_Anyval anyval)) \
   X(hard_copy, pdl*, ( pdl* )) \
   X(converttype, pdl_error, ( pdl*, int )) \
   X(smalloc, void*, ( STRLEN )) \
