@@ -1,12 +1,7 @@
 package PDL::Graphics::OpenGL::Perl::OpenGL;
 
-BEGIN {
-   use PDL::Config;
-   if ($PDL::Config{USE_POGL}) {
-      eval "use OpenGL $PDL::Config{POGL_VERSION} qw()";
-      use OpenGL::Config;
-   }
-}
+use OpenGL ();
+use OpenGL::Config;
 
 BEGIN {
    eval 'OpenGL::ConfigureNotify()';
@@ -28,6 +23,8 @@ BEGIN {
          eval "sub OpenGL::Button1Mask         () { (1<<8 ) }";
          eval "sub OpenGL::Button2Mask         () { (1<<9 ) }";
          eval "sub OpenGL::Button3Mask         () { (1<<10) }";
+         eval "sub OpenGL::Button4Mask         () { (1<<11) }";  # scroll wheel
+         eval "sub OpenGL::Button5Mask         () { (1<<12) }";  # scroll wheel
          eval "sub OpenGL::ButtonMotionMask    () { (1<<13) }";
          eval "sub OpenGL::ExposureMask        () { (1<<15) }";
          eval "sub OpenGL::StructureNotifyMask    { (1<<17) }";
@@ -113,25 +110,41 @@ my (@winObjects) = ();
 #
 #use fields qw/Display Window Context Options GL_Vendor GL_Version GL_Renderer/;
 
-=head2 new($class,$options,[$window_type])
-
-Returns a new OpenGL object with attributes specified in the options
-field, and of the 3d window type, if specified.  These attributes are:
+=head2 new
 
 =for ref
 
-  x,y - the position of the upper left corner of the window (0,0)
-  width,height - the width and height of the window in pixels (500,500)
-  parent - the parent under which the new window should be opened (root)
-  mask - the user interface mask (StructureNotifyMask)
-  attributes - attributes to pass to glXChooseVisual
+Returns a new OpenGL object.
+
+=for usage
+
+  new($class,$options,[$window_type])
+
+  Attributes are specified in the $options field; the 3d $window_type is optionsl. The attributes are:
+
+=over
+
+=item x,y - the position of the upper left corner of the window (0,0)
+
+=item width,height - the width and height of the window in pixels (500,500)
+
+=item parent - the parent under which the new window should be opened (root)
+
+=item mask - the user interface mask (StructureNotifyMask)
+
+=item attributes - attributes to pass to glXChooseVisual
+
+=back
 
 Allowed 3d window types, case insensitive, are:
 
-=for ref
+=over
 
-  glut - use Perl OpenGL bindings and GLUT windows (no Tk)
-  x11  - use Perl OpenGL (POGL) bindings with X11 (disabled)
+=item glut - use Perl OpenGL bindings and GLUT windows (no Tk)
+
+=item x11  - use Perl OpenGL (POGL) bindings with X11 (disabled)
+
+=back
 
 =cut
 
@@ -211,9 +224,7 @@ sub new {
       if(defined($class_or_hash->{Options})){
          return bless $self,ref($class_or_hash);
       }else{
-         foreach(keys %$self){
-            $class_or_hash->{$_} = $self->{$_};
-         }
+         @$class_or_hash{keys %$self} = values %$self;
          return $class_or_hash;
       }
    }
@@ -254,7 +265,7 @@ sub _pdl_fake_KeyPress {
 }
 
 {
-   my @button_to_mask = (256,512,1024);
+   my @button_to_mask = (1<<8, 1<<9, 1<<10, 1<<11, 1<<12);
    my $fake_mouse_state = 16;  # default have EnterWindowMask set;
    my $last_fake_mouse_state;
 
@@ -316,10 +327,14 @@ sub XPending {
 }
 
 
-=head2 XResizeWindow(x,y)
+=head2 XResizeWindow
 
 OO interface to XResizeWindow
 
+=for usage
+
+  XResizeWindow(x,y)
+
 =cut
 
 sub XResizeWindow {
@@ -403,8 +418,8 @@ Chris Marshall, C<< <devel dot chm dot 01 at gmail.com> >>
 
 =head1 BUGS
 
-Bugs and feature requests may be submitted through the PDL sourceforge
-project page at L<http://sourceforge.net/tracker/?group_id=612> .
+Bugs and feature requests may be submitted through the PDL GitHub
+project page at L<https://github.com/PDLPorters/pdl/issues> .
 
 
 =head1 SUPPORT
@@ -413,7 +428,7 @@ PDL uses a mailing list support model.  The Perldl mailing list
 is the best for questions, problems, and feature discussions with
 other PDL users and PDL developers.
 
-To subscribe see the page at L<http://mailman.jach.hawaii.edu/mailman/listinfo/perldl>
+To subscribe see the page at L<http://pdl.perl.org/?page=mailing-lists>
 
 
 
