@@ -22,16 +22,8 @@
 
 package PDL::Graphics::TriD::Mesh;
 
-BEGIN {
-   use PDL::Config;
-   if ( $PDL::Config{USE_POGL} ) {
-      eval "use OpenGL $PDL::Config{POGL_VERSION} qw(:all)";
-      eval 'use PDL::Graphics::OpenGL::Perl::OpenGL';
-   } else {
-      eval 'use PDL::Graphics::OpenGL';
-   }
-}
-
+use OpenGL qw(:all);
+use PDL::Graphics::OpenGL::Perl::OpenGL;
 use PDL::LiteF;
 @ISA=qw/PDL::Graphics::TriD::Object/;
 
@@ -49,7 +41,7 @@ sub new {
 		Data => $data,
 	};
 	PDL::Primitive::axisvalues($this->{Vertices}->slice('(0),:,:'));
-	PDL::Primitive::axisvalues($this->{Vertices}->slice('(1),:,:')->xchg(0,1));
+	PDL::Primitive::axisvalues($this->{Vertices}->slice('(1),:,:')->transpose);
 	PDL::Ops::assgn($this->{Data},$this->{Vertices}->slice('(2),:,:'));
 	bless $this,$type;
 }
@@ -60,10 +52,10 @@ sub printdims {print $_[0].": ".(join ', ',$_[1]->dims)," and ",
 sub get_boundingbox {
 	my($this) = @_;
 	my $foo = PDL->zeroes(6)->double;
-	$a = $this->{Vertices}; printdims "A",$a;
-	$b = $a->thread(0); printdims "B",$b;
-	$c = $b->clump(-1); printdims "C",$c;
-	$d = $c->unthread(1); printdims "D",$d;
+	$A = $this->{Vertices}; printdims "A",$A;
+	$B = $x->thread(0); printdims "B",$B;
+	$C = $y->clump(-1); printdims "C",$C;
+	$D = $c->unthread(1); printdims "D",$D;
 
 	$this->{Vertices}->thread(0);
 
